//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef PHTRIANGLE_HPP
#define PHTRIANGLE_HPP
//------------------------------------------------------------------------------
#include "PhysicObject.hpp"
//------------------------------------------------------------------------------
namespace En
{
/// LUA_HEADER_COPY_ON ///
/** A physic object representing a triangle created using Group::CreatePhTriangle. */
/// LUA_HEADER_COPY_OFF ///
//------------------------------------------------------------------------------
class PhTriangle : public PhysicObjectLine
{
public:
    PhTriangle(Group *parent, int phGroupId, const std::string &name, const Vector2D &pt1, const Vector2D &pt2, const Vector2D &pt3);
    static PhTriangle *Create(Group *parent, int phGroupId, const std::string &name, const Vector2D &pt1, const Vector2D &pt2, const Vector2D &pt3) {return new PhTriangle(parent, phGroupId, name, pt1, pt2, pt3);}

    /// LUA_EXPORT_ON(PhTriangle, PhysicObject, ProtectedPtr<Object>) ///
    // Access point
    /** Get the point \\a nb of the triangle (0 <= nb < 3) */
    Vector2D GetPoint(int nb) const;
    /** Set the point \\a nb of the triangle (0 <= nb < 3) */
    void SetPoint(int nb, const Vector2D &pt);

    /// LUA_EXPORT_OFF ///

    // No Lua
    /** Create a clone of the object, change the parent and return an handler to the clone. */
    virtual PhTriangle *GetRawCloneToGroup(Group *newParent) const {PhTriangle *obj = new PhTriangle(*this); obj->RegisterClone(newParent); return obj;}

    // Type
    ADD_GET_TYPE_FN(OBJ_TYPE_PHYSIC_TRIANGLE)

    virtual std::vector<Vector2D> GetAllPoints() const;
protected:

    Vector2D m_pts[3];
};
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
#endif // PHTRIANGLE_HPP
