------------------------------------------------------------
-- Events
------------------------------------------------------------

-- Out of screen
function gameOnObjectOutScreen(outline, object, position)
	if object:GetName() ~= "phLaser" then
		local parent = object:GetParent()
	
		if parent:UserDataExists("eventToDestroy") then -- We destroy the event
			parent:GetUserData("eventToDestroy"):Destroy()
		end
	
		parent:Destroy() -- We destroy the group
	end
end

-- Bullet intersection
function gameOnBulletShock(phBullet, phEnemy, hitPos)
	if phEnemy:GetPhysicGroup() == phGroupEnemies then
		-- Enemy fn
		phEnemy:GetParent():GetUserData("OnHitFn")(phBullet, phEnemy, hitPos)
		
		RefreshLevel();
	end
end

-- Shield fluctuation
function gameOnShieldFluctuation(ratio, chrono)
	if life == 0 then -- Dead
		return
	end

	shieldSprite.color = Color(255, 255, 255, (1 - ratio) * (shieldFluctuationMax - shieldFluctuationMin) + shieldFluctuationMin)
end

-- Shield Shock
function gameOnShieldShock(phShield, phEnemy, hitPos)
	phEnemy:GetParent():GetUserData("OnShieldShockFn")(phShield, phEnemy, hitPos)
end

-- Mouse move event
function gameOnMouseMove(position, event)
	if life == 0 then -- Dead
		return
	end

	newRotation = gunGroup.rotation - position.x / 250
	newRotation = math.min(newRotation, math.pi / 2)
	newRotation = math.max(newRotation, - math.pi / 2)
	
	gunGroup.rotation = newRotation
	
	-- 08_camera
	gameDesignCamera:SetRotation(newRotation)
	
	-- Design pos
	local angleRatio = (newRotation + math.pi / 2) / math.pi
	
	-- 03_city
	gameDesignCity:SetPositionX(angleRatio * -3)

	-- 04_ground
	gameDesignGround:SetPositionX(angleRatio * -5)
	
	-- 05_fence :
	gameDesignFence:SetPositionX(angleRatio * -6)
end

function gameLaserDestroyTimer(event)
	-- Create laser damage
	local orientation = event:GetUserData("orientation")
	
	local grp = gameScreen:CreateGroup("laserDamageGrp")
	grp:SetUserData("damage", 3000)

	local ph = grp:CreatePhLine(phGroupBullets, "phLaser", gunGroup:GetPosition() + orientation * tex06aGun:GetHeight(), gunGroup:GetPosition() + orientation * math.sqrt(screenSize.x * screenSize.x + screenSize.y * screenSize.y))
	ph:SetEventFunction(gameOnBulletShock)

	event:Destroy()
	
	-- Stop laser flow
	particleLaser:SetFlow(0)
	
	-- Second step : destoy the group
	local event = gameScreen:RegisterTimer(function(event)
												-- Skip the frame and destroy the group
												if event:UserDataExists("firstTime") then
													event:GetUserData("grp"):Destroy()
													event:Destroy()
												else
													event:SetUserData("firstTime", true)
												end
											end, 0.2, true)
	event:SetUserData("grp", grp)
end

-- Laser bt
function gameOnLaserBt(keyName, isPressed, event)
	if canUseLaser and previousLaserTime >= laserWaitTime then
		if life == 0 then -- Dead
			return
		end
	
		-- Laser pos
		orientation = Vector2D(- math.sin(gunGroup.rotation), - math.cos(gunGroup.rotation))
		particleLaser:SetEmitterDirection(orientation)
		particleLaser:SetPosition(gunGroup:GetPosition() + orientation * (tex06aGun:GetHeight()))
		particleLaser:SetFlow(1000)
		
		-- Events
		local event = gameScreen:RegisterTimer(gameLaserDestroyTimer, 0.15, true)
		event:SetUserData("orientation", orientation)
		
		-- Reset time
		previousLaserTime = 0
		
		-- Sound
		gameScreen:PlaySound(sndLaser)
	end
end

-- Freeze bt
function gameOnFreezeBt(keyName, isPressed, event)
	if canUseFreezeWeapon and previousFreezeWeaponTime >= freezeWeaponWaitTime then
		if life == 0 then -- Dead
			return
		end
	
		-- Freeze bonus
		local objTbl = enemyContainer:GetChildTable()
		for i = 1, #objTbl do
			local tmp =  objTbl[i]
			
			if tmp:UserDataExists("points") then -- Enemy
				local currentSpeed = math.sqrt(tmp:GetSpeed().x * tmp:GetSpeed().x + tmp:GetSpeed().y * tmp:GetSpeed().y)
			
				if currentSpeed > minFreezeSpeed then -- On peut encore ralentir
					local ratio = math.min(freezeWeaponSlowDown, currentSpeed / minFreezeSpeed) -- On prend le ratio pour ralentir au max a minSpeed
					tmp:SetSpeed(tmp:GetSpeed() / ratio)
				end
				
				-- Nuclear shell explose				
				if freezeWeaponKillSmallMissile and tmp:GetUserData("enemyType") == "nuclearShell" then
					tmp:GetUserData("OnHitFn")(destroyingWeapon:FindByName("phLaser"), tmp:FindByName("ph"), tmp:GetPosition())
				end
				
				if freezeWeaponResetVaccum and tmp:GetUserData("enemyType") == "vaccum" then
					if tmp:UserDataExists("eventToDestroy") then
						-- We destroy the event
						tmp:GetUserData("eventToDestroy"):Destroy()
						tmp:SetUserData("eventToDestroy", nil)
						
						-- Rechoc shield
						tmp:GetUserData("OnShieldShockFn")(gameScreen:FindByNamePhLine("phShield"), tmp:FindByName("ph"), tmp:GetPosition())
					end
				end
				
				if freezeWeaponMakeBonusFall and tmp:GetUserData("enemyType") == "bonus" then
					local phPara = tmp:FindByName("phPara")

					if phPara ~= nil then -- Le bonus a tjr un parachute
						tmp:GetUserData("OnHitFn")(destroyingWeapon:FindByName("phLaser"), tmp:FindByName("phPara"), tmp:GetPosition())
					end
				end				
			end
		end
		
		-- Test deflag
		CreateDeflagration(gunGroup:GetGlobalPosition() - Vector2D(0, 75))
		
		-- Reset time
		previousFreezeWeaponTime = 0
		
		-- Sound
		gameScreen:PlaySound(sndFreezeWeapon)
	end
end

-- On frame
function gameOnFrame(frameTime, event)
	if life == 0 then -- Dead
		return
	end
	
	-- Time played
	totalPlayTime = totalPlayTime + frameTime

	-- Regen life
	if life ~= maxLife then	
		life = life + frameTime * lifeRecoverySpeed
		lifeRecoverySpeed = math.min(lifeRecoverySpeed + lifeRecoverySpeed * lifeRecoveryAcceleration * frameTime, lifeRecoveryMaxSpeed) -- Double each second
	
		life = math.min(maxLife, life)
		
		RefreshLifeText()
	end
	
	-- Shoot	
	if previousShootTime >= shootWaitTime and IsKeyPressed("MouseLeft") then
		local bullet = stockBullet:CloneToGroup(gameScreen)
		
		rotation = gunGroup:GetRotation()
		bullet:SetRotation(rotation)
		gunSize = tex06aGun:GetHeight()
		orientation = Vector2D(- math.sin(rotation), - math.cos(rotation))
		gunTopPosition = gunGroup:GetPosition() + orientation * gunSize
		bullet:SetPosition(gunTopPosition)
		bullet:SetSpeed(orientation * bulletSpeed)
		
		shootFlash:SetPosition(gunTopPosition)
		shootFlash:SetTank(1)
		
		-- Sound
		gameScreen:PlaySound(sndShot)
		
		-- Wait time
		previousShootTime = 0
	elseif previousShootTime < shootWaitTime then
		previousShootTime = math.min(previousShootTime + frameTime, shootWaitTime)
		
		GroupSetAmmoProgress(bulletAmmoCircle, previousShootTime / shootWaitTime)
		
		-- Pos
		orientation = Vector2D(- math.sin(rotation), - math.cos(rotation))
		gunGroup:FindByName("gunSpr"):SetPosition(Vector2D(0, - 20 * ((shootWaitTime - previousShootTime) / shootWaitTime)))
	end

	-- Laser
	if canUseLaser and previousLaserTime < laserWaitTime then
		previousLaserTime = math.min(previousLaserTime + frameTime, laserWaitTime)
		GroupSetAmmoProgress(laserAmmoCircle, previousLaserTime / laserWaitTime)
		
		if previousLaserTime == laserWaitTime and useLaserAlarm then
			gameScreen:PlaySound(sndLaserReady)
		end
	end

	-- Freeze weapon
	if canUseFreezeWeapon and previousFreezeWeaponTime < freezeWeaponWaitTime then
		previousFreezeWeaponTime = math.min(previousFreezeWeaponTime + frameTime, freezeWeaponWaitTime)
		GroupSetAmmoProgress(freezeWeaponAmmoCircle, previousFreezeWeaponTime / freezeWeaponWaitTime)
		
		if previousFreezeWeaponTime == freezeWeaponWaitTime and useFreezeWeaponAlarm then
			gameScreen:PlaySound(sndFreezeWeaponReady)
		end
	end
end

-- Fluctuation arrivé enemis
function spawnFn(x)
	local basePeriod = 10
	
	local y = math.sin(x / basePeriod) + math.sin(x / 3) / 4 + math.sin(x) / 5
	y = y / (1 + 1/4 + 1/5) -- appartient à [-1; 1]
	y = (y + 1) / 2 -- appartient à [0; 1]
	
	return y
end

-- Enemy timer
function gameOnTimer(event)
	if life == 0 then -- Dead
		return
	end
	
	local currentEnemyNb = enemyContainer:GetChildCount()
	local targetEnemyNb = (1.8 * level + 2) * (0.5 + spawnFn(totalPlayTime))
	
	if currentEnemyNb < targetEnemyNb then
		for i = 1, targetEnemyNb - currentEnemyNb do
			getSpawnFn(level)(enemyContainer)
		end
	end
end

-- Store event
function gameOnStore(keyName, isPressed, event)
	if life == 0 then -- Dead
		return
	end

	RefreshStore()
	createMovingBlockAnim(storeScreen:FindByName("storeGrp"), Vector2D(screenSize.x, 0), Vector2D(0, 0), 1.5, false)

	ChangeScreen(storeScreen)
end

-- Event quit
function gameOnQuit(keyName, isPressed, event)
	if life == 0 then -- Dead
		return
	end

	ChangeScreen(menuScreen)
end

-- Final explosion -> Death anim
function gameExplosionEffect(ratio, event)
	local isStep2 = event:GetUserData("2nd_step")

	if ratio == 1 then
		if isStep2 then -- Fin			
			event:GetUserData("effect"):Destroy()
			event:Destroy()
		else
			event:SetUserData("2nd_step", true)
			event:Restart()
			
			gameScreen:FindByName("dead_background"):Show(true)
			DestroyAllEnemies()
		end
	end
	
	if isStep2 then
		ratio = 1 - ratio -- Disapear
	end
	
	event:GetUserData("effect"):SetParameter("ratio", ratio)
end

function gameCreateDeathAnim()
	-- Disable backToGameBt in menu
	buttonEnable(menuScreen:FindByNameGroup("backBt"), false)
	buttonEnable(menuScreen:FindByNameGroup("storeBt"), false)

	-- Explosions
	local timeSpan = 1
	local currentTime = 0
	
	for i = 1, 15 do
		currentTime = currentTime + timeSpan
		gameScreen:RegisterTimer(function(event)
									CreateExplosion(Vector2D(math.random() * screenSize.x, screenSize.y - math.random() * 100 - 50), math.pi * 2, Vector2D(0, -1), true)
									gameScreen:PlaySound(sndShieldResist)
									event:Destroy()
								end, currentTime, true)
		
		timeSpan = timeSpan / 1.3
	end
	
	local explosionEffect = gameScreen:CreatePostFX("explosionEffect")
	explosionEffect:LoadFromFile("sfx/explosition.sfx")
	explosionEffect:SetZpos(10000)
	--lowLifeEffect:SetParameter("effectDist", 0.2)
	
	local tmpEvt = gameScreen:RegisterChronometer(gameExplosionEffect, 3, false) -- Stopped
	tmpEvt:SetUserData("effect", explosionEffect)
	tmpEvt:SetUserData("2nd_step", false)
	
	-- Start the explosion shader
	local tmpLauncher = gameScreen:RegisterTimer(function(event)
													event:GetUserData("eventToStart"):Start()
													event:Destroy()
												end, 3, true)
	tmpLauncher:SetUserData("eventToStart", tmpEvt)
	
	-- Burst all enemies
	gameScreen:RegisterTimer(function(event)
									local newScorePos = scoreAdd(score, level, scoreTbl)
									saveScoreTbl(scoreTbl)
									RefreshScores(scoreTbl, newScorePos)

									local txt = _("You are dead !") .. "\n" .. _("Score: ") .. tostring(score) .. "\n" .. _("Level: ") .. tostring(level) .. "\n"
									if newScorePos == -1 then
										txt = txt .. _("You are not part of the top 10.")
									else
										txt = txt .. _("You are in the top 10!")
									end
									
									popupDialog(scoreScreen, "dialogOk", txt, function() end)
									
									event:Destroy()
								end, 8.5, true)
	
	-- Show score screen
	gameScreen:RegisterTimer(function(event)
									BurstAllEnemies()
								end, 3.5, true)
end


-- Anim fade
function CreateSpriteFadeOutAnim(parent, position, tex, duration)
	local grp = parent:CreateGroup("animTmpGroup")

	local spr = grp:CreateSprite("animTmpSprite")
	spr:SetImage(tex)
	spr:SetPosition(position)
	
	local evt = parent:GetParentScreen():RegisterChronometer(function(ratio, event)
																	event:GetUserData("spr"):SetColor(Color(255, 255, 255, (1 - ratio) * 255))
																	
																	if ratio == 1 then
																		event:GetUserData("spr"):GetParent():Destroy()
																	end
																end, duration, true)
	evt:SetUserData("spr", spr)
	
	-- auto destroy
	guiLib_addAutoDestroyEvent(grp, evt)
	
	-- Return the group
	return grp
end

-- Shield impact
function CreateShieldImpact(xPos)
	CreateSpriteFadeOutAnim(gameScreen, Vector2D(xPos - 69, 842) - gameScreen:GetGlobalPosition(), shieldImpactTex, 4)
end

-- Clouds mvt
function gameCloudsMvt(frameTime)
	local speed = 10

	cloudsSprP1:Move(Vector2D(speed * frameTime, 0))
	cloudsSprP2:Move(Vector2D(speed * frameTime, 0))
	
	-- Si un spr sort de l'écran, on le met derrière l'autre
	if cloudsSprP1:GetPosition().x > screenSize.x then
		cloudsSprP1:SetPositionX(cloudsSprP2.x - cloudsSprP1.size.x)
	elseif cloudsSprP2:GetPosition().x > screenSize.x then
		cloudsSprP2:SetPositionX(cloudsSprP1.x - cloudsSprP2.size.x)
	end	
end

