------------------------------------------------------------
-- Spawn list
------------------------------------------------------------


spawnList={}

function addToSpawnList(fn, minLevel, chance)
	local newElement = {}
	newElement = {}
	newElement["fn"] = fn
	newElement["minLevel"] = minLevel
	newElement["chance"] = chance

	table.insert(spawnList, newElement)
end

function getSpawnFn(level)
	local total = 0

	for i = 1, #spawnList do
		if level >= spawnList[i]["minLevel"] then
			total = total + spawnList[i]["chance"]
		end
	end
	
	local chance = math.random() * total
	local currentChance = 0
	local fn
		
	local i = 1
	while currentChance < chance do
		if level >= spawnList[i]["minLevel"] then
			fn = spawnList[i]["fn"]
			currentChance = currentChance + spawnList[i]["chance"]
		end
		
		i = i + 1
	end
	
	return fn
end


------------------------------------------------------------
-- Enemies
------------------------------------------------------------
dofile("scripts/stockMissile.lua")
dofile("scripts/stockNuclearShell.lua")
dofile("scripts/stockPlane.lua")
dofile("scripts/stockBonus.lua")
dofile("scripts/stockShieldVacuum.lua")
dofile("scripts/stockMissilesAlain.lua")
dofile("scripts/stockBombeAlain.lua")

------------------------------------------------------------
-- Bullet
------------------------------------------------------------

-- Bullet
stockBullet = stockScreen:CreateGroup("bullet")
stockBullet:SetUserData("damage", 30)
--tmp = stockBullet:CreatePhLine(phGroupBullets, "phBullet", Vector2D(0, 0), Vector2D(0, -tex_bullet:GetHeight()))
tmp = stockBullet:CreatePhRect(phGroupBullets, "phBullet", Vector2D(- tex_bullet:GetWidth() / 2, -tex_bullet:GetHeight()), Vector2D(tex_bullet:GetWidth(), tex_bullet:GetHeight()))
tmp:SetEventFunction(gameOnBulletShock)

tmpSprite = stockBullet:CreateSprite("spr")
tmpSprite:SetImage(tex_bullet)
tmpSprite.y = - tex_bullet:GetHeight()
tmpSprite.x = - tex_bullet:GetWidth() / 2

-- Reacteur --
model = ParticleModel(0.1, 0.2)
model:AddMutableParam("alpha", 0.7, 0)
model:AddMutableRandomParam("blue", 0.7, 1, 0.1, 0.6)
model:AddMutableRandomParam("red", 0.7, 1, 0.1, 0.6)
model:AddMutableRandomParam("green", 0.7, 1, 0, 0.2)
model:AddMutableRandomParam("size", 0.2, 0.5, 0.8, 1.2)

emitter = ParticleEmitter()
emitter:MakeDisk(Vector2D(0, 0), 4)
--[[emitter:SetAngles(3.14 / 4, 0)
emitter:SetDirection(Vector2D(0, -1))]]

-- Particle
particleObj = stockBullet:CreateParticleSource("particle", model, 20)
particleObj:MakeRenderImages(texParticlePoint, Vector2D(40, 40), true)
particleObj:SetEmitter(emitter)
particleObj:SetFlow(100)
particleObj:SetForce(30, 40)
particleObj:SetFriction(3)
particleObj:SetGravity(Vector2D(0, -15))
particleObj.zPos = -100
particleObj.position = Vector2D(1, 5)


------------------------------------------------------------
-- Progress (*7)
------------------------------------------------------------
stockSlice = stockScreen:CreateGroup("progressSlices")
tmpSprite = stockSlice:CreateSprite("back")
tmpSprite:SetZpos(-10)
tmpSprite:SetImage(texlifeBack)
tmpSprite:SetCenter(Vector2D(texlifeBack:GetWidth() / 2 + 1, texlifeBack:GetHeight() / 2 + 1))
tmpSprite = stockSlice:CreateSprite("slice0")
tmpSprite:SetImage(texProgressCircle)
tmpSprite:SetPosition(Vector2D(0, 0))
tmpSprite:SetCenter(Vector2D(texProgressCircle:GetWidth() / 2, texProgressCircle:GetHeight() / 2))

lifeSliceMax = 7

for i = 1, lifeSliceMax - 1 do
	local tmp = tmpSprite:Clone()
	tmp:SetName("slice" .. tostring(i))
	tmp:SetRotation(math.pi * 2 * (i / lifeSliceMax))
end

function GroupSetProgress(group, progress)
	for i = 0, lifeSliceMax - 1 do
		local tmp = group:FindByNameSprite("slice" .. tostring(i))
		local color = Color(0, 255, 0, 255)
		
		if progress < 0.4 then
			color.g = math.pow(progress / 0.4, 3) * 255
			color.r = 255 - color.g
		end
		
		if progress >= (i + 1) / lifeSliceMax then -- full color
			color.a = 255
		elseif progress <= i / lifeSliceMax then -- Pas atteint
			color.a = 0
		else
			color.a = (progress - (i / lifeSliceMax)) / (1 / lifeSliceMax) * 255
		end
		
		tmp:SetColor(color)
	end
end

------------------------------------------------------------
-- Progress (*4)
------------------------------------------------------------
stockAmmoSlice = stockScreen:CreateGroup("progressAmmoSlices")
tmpSprite = stockAmmoSlice:CreateSprite("slice0")
tmpSprite:SetImage(texProgress4xCircle)
tmpSprite:SetPosition(Vector2D(0, 0))
tmpSprite:SetCenter(Vector2D(texProgress4xCircle:GetWidth(), texProgress4xCircle:GetHeight()))

sliceAmmoMax = 4

for i = 1, sliceAmmoMax - 1 do
	local tmp = tmpSprite:Clone()
	tmp:SetName("slice" .. tostring(i))
	tmp:SetRotation(math.pi * 2 * (i / sliceAmmoMax))
end

function GroupSetAmmoProgress(group, progress)
	for i = 0, sliceAmmoMax - 1 do
		local tmp = group:FindByNameSprite("slice" .. tostring(i))
		local color = Color(0, 255, 0, 255)
		
		if progress < 1 then
			color.g = math.pow(progress, 3) * 255
			color.r = 255 - color.g
		end
		
		if progress >= (i + 1) / sliceAmmoMax then -- full color
			color.a = 255
		elseif progress <= i / sliceAmmoMax then -- Pas atteint
			color.a = 0
		else
			color.a = (progress - (i / sliceAmmoMax)) / (1 / sliceAmmoMax) * 255
		end
		
		tmp:SetColor(color)
	end
end

