------------------------------------------------------------
-- shieldVacuum 1
------------------------------------------------------------

-- Fns
function shieldVacuum_Create(toGroup, forcedPos, enemyLevel)
	-- Clone
	local enemy = stockShieldVacuum:CloneToGroup(toGroup)
	
	if forcedPos == nil then
		-- Pas dans les marges droite et gauche
		local borderMargin = enemy:FindByNameSprite("spr"):GetSize().x / 2
		
		-- Ne peut pas apparaitre trop près du cannon (pas atteignable)
		local gunZone = tex06aGun:GetHeight() * 2 + 20 -- Largeur + 10 de chaque côté
		local maxX = screenSize.x - gunZone - borderMargin * 2
		
		forcedPos = Vector2D(borderMargin + math.random() * maxX, -50)
		
		if forcedPos.x > maxX / 2 + borderMargin then
			forcedPos.x = forcedPos.x + gunZone
		end
	end
	
	enemy:SetPosition(forcedPos)
	
	-- Level
	local texTbl = vaccumTexTbl;
	
	enemyLevel = math.min(enemyLevel, #texTbl)
	enemyLevel = math.max(enemyLevel, 1)
	
	enemy:FindByNameSprite("spr"):SetImage(texTbl[enemyLevel])
	enemy:SetUserData("points", enemyLevel)
	enemy:SetUserData("enemyLevel", enemyLevel)
	
	-- Speed
	enemy:SetSpeed(Vector2D(0, 160 + 20 * enemyLevel)* (0.8 + math.random() * 0.4)) -- Speed +- 40%
	
	-- Return
	return enemy
end

function shieldVacuum_OnHit(phBullet, phEnemy, hitPos)
	enemyGrp = phEnemy:GetParent()
	bulletGrp = phBullet:GetParent()
	
	enemyGrp:SetUserData("life", enemyGrp:GetUserData("life") - bulletGrp:GetUserData("damage"))

	if enemyGrp:GetUserData("life") <= 0 then -- If the enemy is dead
		-- Enemy fn
		-- Explosion
		CreateExplosion(hitPos, math.pi * 3/5, bulletGrp:GetSpeed())

		-- Play Snd
		gameScreen:PlaySound(sndEnemyBurstTbl[math.ceil(#sndEnemyBurstTbl * math.random())])
		
		-- Points
		score = score + enemyGrp:GetUserData("points")
		
		if enemyGrp:UserDataExists("eventToDestroy") then -- We destroy the event
			enemyGrp:GetUserData("eventToDestroy"):Destroy()
		end
		
		enemyGrp:SetUserData("destroyedNextFrame", true)
		enemyGrp:Destroy() -- We destroy the group
	else -- The enemy is style alive
		gameScreen:PlaySound(sndEnemyResist)
	end
	
	if phBullet:GetName() ~= "phLaser" then
		phBullet:GetParent():Destroy() -- We destroy the group
	end
end

function shieldVacuum_OnShieldShock(phShield, phEnemy, hitPos)
	local enemyGrp = phEnemy:GetParent()
	
	--// If the object will be destroyed next frame => We don't create any event (will cause seg fault)
	if enemyGrp:UserDataExists("destroyedNextFrame") then
		return
	end
	
	-- Test si aucun event est enregistré pour ce mode
	if (enemyGrp:UserDataExists("eventToDestroy")) and (enemyGrp:GetUserData("eventToDestroy") ~= nil) then
		printErrorConsole("Shield Vaccum : eventToDestroy already exists on shieldVacuum_OnShieldShock !!")
		Quit()
		return -- Quite a la fin de la frame
	end
	
	-- Pos & speed
	enemyGrp:SetSpeed(Vector2D(0, 0))
	enemyGrp:SetPositionY(phShield:GetPosition().y - 2 - enemyGrp:FindByName("ph"):GetPoint(1).y)
	
	-- Plus de stabilisateurs
	enemyGrp:FindByName("particle2"):SetFlow(0);
	enemyGrp:FindByName("particle"):SetFlow(0);
	
	-- Start vaccum wait chrono
	local enemyLevel = enemyGrp:GetUserData("enemyLevel")
	local event = gameScreen:RegisterChronometer(shieldVacuum_OnWaitVaccumChrono, 6 - enemyLevel * 1.2, true)
	event:SetUserData("grp", enemyGrp)
	
	-- Register event
	enemyGrp:SetUserData("eventToDestroy", event)
end

function shieldVacuum_OnWaitVaccumOn(frameTime, event)
	ChangeLife(-frameTime * 100)
	
	-- Son
	local period = 0.8
	local lastSoundTime = event:GetUserData("lastSoundTime")
	if lastSoundTime > period then
		event:GetUserData("grp"):GetParentScreen():PlaySound(sndVaccum)
		lastSoundTime = lastSoundTime % period
	end
	
	lastSoundTime = lastSoundTime + frameTime
	event:SetUserData("lastSoundTime", lastSoundTime)
end

function shieldVacuum_OnWaitVaccumChrono(ratio, event)
	local enemyGrp = event:GetUserData("grp")

	-- If the ratio is 1 => We start the vaccum
	if ratio == 1 then	
		-- We destroy the previous event
		event:Destroy()
		
		-- Start vaccum on chrono
		local newEvent = gameScreen:RegisterFrameEvent(shieldVacuum_OnWaitVaccumOn)
		newEvent:SetUserData("grp", enemyGrp)
		newEvent:SetUserData("lastSoundTime", 0)
		
		-- Register event
		enemyGrp:SetUserData("eventToDestroy", newEvent)
		
		-- Particle
		local particle1 = enemyGrp:FindByName("particle");
		local particle2 = enemyGrp:FindByName("particle2");
		
		particle1:SetPosition(Vector2D(-18, -36))
		particle1:SetEmitterDirection(Vector2D(-4, -10))
		
		particle2:SetPosition(Vector2D(23, -32))
		particle2:SetEmitterDirection(Vector2D(10, -7))
		
		enemyGrp:FindByName("particle2"):SetFlow(30);
		enemyGrp:FindByName("particle"):SetFlow(30);
		
		enemyGrp:FindByNameParticleSource("particle"):SetForce(150, 500)
		
		-- Snd
		enemyGrp:GetParentScreen():PlaySound(sndVaccum);
	end
	
	-- Progress bar
	local barWidth = 9 - (-11)
	enemyGrp:FindByNameShape("progressBar"):MakeRectangle(Vector2D(-11, -11), Vector2D(-11 + barWidth * ratio, -5), Color(0, 255, 0), 0, Color(0, 0, 0))
	
	-- Change color
	enemyGrp:FindByNameSprite("spr"):SetColor(Color(255, 255 - ratio * 100, 255 - ratio * 100))
end

-- Save
addToSpawnList(function(toGroup, forcedPos) shieldVacuum_Create(toGroup, forcedPos, 1); end, 4, 1.25)
addToSpawnList(function(toGroup, forcedPos) shieldVacuum_Create(toGroup, forcedPos, 1); end, 8, 0.75)
addToSpawnList(function(toGroup, forcedPos) shieldVacuum_Create(toGroup, forcedPos, 2); end, 10, 2)
addToSpawnList(function(toGroup, forcedPos) shieldVacuum_Create(toGroup, forcedPos, 3); end, 19, 2)

-- Vaccum
stockShieldVacuum = stockScreen:CreateGroup("enemyVaccum")
stockShieldVacuum:SetUserData("life", 20)
stockShieldVacuum:SetUserData("damage", 30)
stockShieldVacuum:SetUserData("enemyType", "vaccum")

-- Functions
stockShieldVacuum:SetUserData("OnHitFn", shieldVacuum_OnHit)
stockShieldVacuum:SetUserData("OnShieldShockFn", shieldVacuum_OnShieldShock)

-- Sprite
tmpSprite = stockShieldVacuum:CreateSprite("spr")
tmpSprite:SetImage(vaccumTexTbl[1])
--tmpSprite:FlipX(true)
tmpSprite.y = - tmpSprite:GetSize().y / 2
tmpSprite.x = - tmpSprite:GetSize().x / 2

-- Physic
--stockShieldVacuum:CreatePhTriangle(phGroupEnemies, "ph", Vector2D(- tmpSprite:GetSize().x / 2, - tmpSprite:GetSize().y), Vector2D(tmpSprite:GetSize().x / 2, - tmpSprite:GetSize().y), Vector2D(0, 0))
stockShieldVacuum:CreatePhTriangle(phGroupEnemies, "ph", Vector2D(- tmpSprite:GetSize().x / 2, tmpSprite:GetSize().y / 2 - 13), Vector2D(tmpSprite:GetSize().x / 2, tmpSprite:GetSize().y / 2 - 13), Vector2D(0, - tmpSprite:GetSize().y / 2))


-- Reacteur --
model = ParticleModel(0.3, 0.7)
model:AddMutableParam("alpha", 0.7, 0)
model:AddMutableRandomParam("blue", 0.3, 0.5, 0, 0.1)
model:AddMutableRandomParam("red", 0.8, 1, 0.7, 1)
model:AddMutableRandomParam("green", 0.8, 1, 0, 0.2)
model:AddMutableRandomParam("size", 0.7, 1, 1.5, 2)

emitter = ParticleEmitter()
emitter:MakeDisk(Vector2D(0, 0), 2)
emitter:SetAngles(3.14 / 6, 0)
emitter:SetDirection(Vector2D(0, 1))

-- Particle
particleObj = stockShieldVacuum:CreateParticleSource("particle", model, 21)
particleObj:MakeRenderImages(texParticlePoint, Vector2D(30, 30), true)
particleObj:SetEmitter(emitter)
particleObj:SetFlow(30)
particleObj:SetForce(300, 400)
particleObj:SetFriction(3)
particleObj:SetGravity(Vector2D(0, 10))
particleObj.position = Vector2D(-37, 34)
particleObj.zPos = -200

-- 2é patte
local tmp = particleObj:Clone()
tmp:SetName("particle2")
tmp:SetPosition(Vector2D(39, 34))
tmp.zPos = -200

-- Progress bar
local tmp = stockShieldVacuum:CreateShape("progressBar")
particleObj.zPos = 200
